==> gitlab-tools-v0.2.0.tar.gz: gitlab-tools/bin/glx-audit.sh <==
#!/bin/bash
# shellcheck disable=SC1090
#
# gitlab-tools:bin/glx-audit.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -gr PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1

function usage {
  cat <<EOF
Audit for GitLab

Usage:

  ${BASH_SOURCE[0]} --help
  ${BASH_SOURCE[0]} --directory AUDIT_DESTINATION_FOLDER

EOF
}

function mk_relative_link {
  local -r target="$1"
  local -r link_name="$2"
  local    relative_target=

  if [ -z "${target}" ]; then
    echofatal '*** mk_relative_link: target value is empty.'
    exit 1
  fi
  if [ -z "${link_name}" ]; then
    echofatal '*** mk_relative_link: link_name value is empty.'
    exit 1
  fi

  if [ -f "${link_name}" ]; then
    rm "${link_name}" # avoid link resolution
  fi

  cat <<EOF >&2
  relative_target="$( realpath --no-symlinks --relative-to="$( dirname "${link_name}" )" "${target}" )"
EOF
  relative_target="$( realpath --no-symlinks --relative-to="$( dirname "${link_name}" )" "${target}" )"
  if [ -z "${relative_target}" ]; then
    echofatal "*** mk_relative_link: relative_target value is empty. target='${target}' / link_name='${link_name}'"
    exit 1
  fi

  ln -sv --force "${relative_target}" "${link_name}" >&2
}

function build_audit_folder {
  local -r audit_folder="$1"
  local -r file_type="$2"

  if [ -z "${audit_folder}" ]; then
    echoerror 'build_audit_folder: audit_folder value is missing'
    exit 1
  fi
  if [ -z "${file_type}" ]; then
    echoerror 'build_audit_folder: file_type value is missing'
    exit 1
  fi

  local -r folder="${audit_folder}/${file_type}"

  if [ ! -d "${folder}" ]; then
    mkdir -vp "${folder}" >&2 || exit $?
  fi

  echo "${folder}"
}

function build_audit_file {
  local -r audit_folder="$1"
  local -r file_type="$2"
  local -r file_name="$3"

  if [ -z "${audit_folder}" ]; then
    echofatal "*** build_audit_file: audit_folder value is missing"
    exit 1
  fi
  if [ -z "${file_type}" ]; then
    echofatal "*** build_audit_file: file_type value is missing"
    exit 1
  fi
  if [ -z "${file_name}" ]; then
    echofatal "*** build_audit_file: file_name value is missing"
    exit 1
  fi

  local file=
  local parent=

  file="$(
    build_audit_folder "${audit_folder}" "${file_type}"
    )/${file_name}.json" || exit 1
  parent="$( dirname "${file}" )" || exit 1

  if [ ! -d "${parent}" ]; then
    mkdir "${parent}"
  fi

  echo "${file}"
}

function get_groups_json {
  "${PROJECT_HOME}/bin/glt-group.sh" list || exit 1
}

function get_projects_json {
  "${PROJECT_HOME}/bin/glt-project.sh" list || exit 1
}

function audit_groups_json_filter {
  jq "[ . | { ${GITLAB_AUDIT_FILTER_FOR_GROUP} } ]"
}

function audit_groups_configuration {
  local -r audit_folder="$1"
  local    group_json=
  local    group_path=
  local    group_audit_json=
  local    audit_file=
  local    path_link=

  while read -r group_json ; do
    group_id="$( jq -r '.id' <<<"${group_json}" )" || exit 1
    group_path="$( jq -r '.path' <<<"${group_json}" )" || exit 1

    if [ -z "${group_path}" ]; then
      echowarn "*** Error: can not retrieve configuration for group '${group_id}'" >&2
    else
      group_audit_json="$( audit_groups_json_filter <<<"${group_json}" )" || exit 1

      audit_file="$( build_audit_file "${audit_folder}" 'groups_by_id' "${group_id}" )" || exit 1
      path_link="$( build_audit_file "${audit_folder}" 'groups_by_path' "${group_path}" )" || exit 1

      echoinfo "* audit group ${group_id} / ${group_path}"
      echo "${group_audit_json}" > "${audit_file}"

      mk_relative_link "${audit_file}" "${path_link}"
    fi
  done < <( get_groups_json | jq -c '.[]' )
}

function audit_projects_json_filter {
  jq "[ . | { ${GITLAB_AUDIT_FILTER_FOR_PROJECT} } ]"
}

function audit_projects_configuration {
  local -r audit_folder="$1"
  local    project_json=
  local    project_audit_json=
  local    project_path=
  local    project_fullpath=
  local    audit_file=
  local    path_link=
  local    fullpath_link=

  while read -r project_json ; do
    project_id="$( jq -r '.id' <<<"${project_json}" )" || exit 1
    project_path="$( jq -r '.path' <<<"${project_json}" )" || exit 1
    project_fullpath="$( jq -r '.path_with_namespace' <<<"${project_json}" )" || exit 1

    if [ -z "${project_path}" ]; then
      echowarn "*** Error: can not retrieve configuration for project '${project_id}'" >&2
    else
      project_audit_json="$( audit_projects_json_filter <<<"${project_json}" )" || exit 1

      audit_file="$( build_audit_file "${audit_folder}" 'projects_by_id' "${project_id}" )" || exit 1
      path_link="$( build_audit_file "${audit_folder}" 'projects_by_path' "${project_path}" )" || exit 1
      fullpath_link="$( build_audit_file "${audit_folder}" 'projects_by_path_with_namespace' "${project_fullpath}" )" || exit 1

      echoinfo "* audit project ${project_id} / ${project_path}"
      echo "${project_audit_json}" > "${audit_file}"

      mk_relative_link "${audit_file}" "${path_link}"
      mk_relative_link "${audit_file}" "${fullpath_link}"
    fi
  done < <( get_projects_json | jq -c '.[]' )
}

function do_audit {
  local -r audit_folder_home="$1"

  #
  audit_groups_configuration "${audit_folder_home}" || exit 1
  #
  audit_projects_configuration "${audit_folder_home}" || exit 1

  #audit_users_configuration ? "${audit_folder_home}" || exit 1
  #audit_merge_request ? "${audit_folder_home}" || exit 1
  #audit_merge_request_configuration ? "${audit_folder_home}" || exit 1
  #audit_merge_request ? "${audit_folder_home}" || exit 1
}

function main {
  local audit_folder_home=

  while [ $# -gt 0 ] ; do
    local param="$1"
    shift

    case "${param}" in
      '-d'|'--directory')
        audit_folder_home="$1"
        shift
        ;;

      *)
        # unknown option
        echofatal "Unknown parameter: '${param}'" >&2
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${audit_folder_home}" ]; then
    echofatal 'audit directory is missing'
    usage >&2
    exit 1
  fi

  # shellcheck disable=SC1090
  source "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" || exit 1
  # shellcheck disable=SC1090
  source "${GITLAB_BASH_API_PATH}/post.sh" || exit 1
  #source "${GITLAB_BASH_API_PATH}/api/gitlab-bash-api-project.sh"

  if ! is_variable_define_and_not_empty_safe 'GITLAB_AUDIT_FILTER_FOR_GROUP' ; then
    echofatal 'GITLAB_AUDIT_FILTER_FOR_GROUP is not define.'
    exit 1
  fi

  if ! is_variable_define_and_not_empty_safe 'GITLAB_AUDIT_FILTER_FOR_PROJECT' ; then
    echofatal 'GITLAB_AUDIT_FILTER_FOR_PROJECT is not define.'
    exit 1
  fi

  do_audit "${audit_folder_home}"
}

main "$@"

==> gitlab-tools-v0.2.1.tar.gz: gitlab-tools/bin/glx-audit.sh <==
#!/bin/bash
# shellcheck disable=SC1090
#
# gitlab-tools:bin/glx-audit.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -gr PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1

function usage {
  cat <<EOF
Audit for GitLab

Usage:

  ${BASH_SOURCE[0]} --help
  ${BASH_SOURCE[0]} --directory AUDIT_DESTINATION_FOLDER

EOF
}

function mk_relative_link {
  local -r target="$1"
  local -r link_name="$2"
  local    relative_target=

  if [ -z "${target}" ]; then
    echofatal '*** mk_relative_link: target value is empty.'
    exit 1
  fi
  if [ -z "${link_name}" ]; then
    echofatal '*** mk_relative_link: link_name value is empty.'
    exit 1
  fi

  if [ -f "${link_name}" ]; then
    rm "${link_name}" # avoid link resolution
  fi

  cat <<EOF >&2
  relative_target="$( realpath --no-symlinks --relative-to="$( dirname "${link_name}" )" "${target}" )"
EOF
  relative_target="$( realpath --no-symlinks --relative-to="$( dirname "${link_name}" )" "${target}" )"
  if [ -z "${relative_target}" ]; then
    echofatal "*** mk_relative_link: relative_target value is empty. target='${target}' / link_name='${link_name}'"
    exit 1
  fi

  ln -sv --force "${relative_target}" "${link_name}" >&2
}

function build_audit_folder {
  local -r audit_folder="$1"
  local -r file_type="$2"

  if [ -z "${audit_folder}" ]; then
    echoerror 'build_audit_folder: audit_folder value is missing'
    exit 1
  fi
  if [ -z "${file_type}" ]; then
    echoerror 'build_audit_folder: file_type value is missing'
    exit 1
  fi

  local -r folder="${audit_folder}/${file_type}"

  if [ ! -d "${folder}" ]; then
    mkdir -vp "${folder}" >&2 || exit $?
  fi

  echo "${folder}"
}

function build_audit_file {
  local -r audit_folder="$1"
  local -r file_type="$2"
  local -r file_name="$3"

  if [ -z "${audit_folder}" ]; then
    echofatal "*** build_audit_file: audit_folder value is missing"
    exit 1
  fi
  if [ -z "${file_type}" ]; then
    echofatal "*** build_audit_file: file_type value is missing"
    exit 1
  fi
  if [ -z "${file_name}" ]; then
    echofatal "*** build_audit_file: file_name value is missing"
    exit 1
  fi

  local file=
  local parent=

  file="$(
    build_audit_folder "${audit_folder}" "${file_type}"
    )/${file_name}.json" || exit 1
  parent="$( dirname "${file}" )" || exit 1

  if [ ! -d "${parent}" ]; then
    mkdir "${parent}"
  fi

  echo "${file}"
}

function get_groups_json {
  "${PROJECT_HOME}/bin/glt-group.sh" list || exit 1
}

function get_projects_json {
  "${PROJECT_HOME}/bin/glt-project.sh" list || exit 1
}

function audit_groups_json_filter {
  jq "[ . | { ${GITLAB_AUDIT_FILTER_FOR_GROUP} } ]"
}

function audit_groups_configuration {
  local -r audit_folder="$1"
  local    group_json=
  local    group_path=
  local    group_audit_json=
  local    audit_file=
  local    path_link=

  while read -r group_json ; do
    group_id="$( jq -r '.id' <<<"${group_json}" )" || exit 1
    group_path="$( jq -r '.path' <<<"${group_json}" )" || exit 1

    if [ -z "${group_path}" ]; then
      echowarn "*** Error: can not retrieve configuration for group '${group_id}'" >&2
    else
      group_audit_json="$( audit_groups_json_filter <<<"${group_json}" )" || exit 1

      audit_file="$( build_audit_file "${audit_folder}" 'groups_by_id' "${group_id}" )" || exit 1
      path_link="$( build_audit_file "${audit_folder}" 'groups_by_path' "${group_path}" )" || exit 1

      echoinfo "* audit group ${group_id} / ${group_path}"
      echo "${group_audit_json}" > "${audit_file}"

      mk_relative_link "${audit_file}" "${path_link}"
    fi
  done < <( get_groups_json | jq -c '.[]' )
}

function audit_projects_json_filter {
  jq "[ . | { ${GITLAB_AUDIT_FILTER_FOR_PROJECT} } ]"
}

function audit_projects_configuration {
  local -r audit_folder="$1"
  local    project_json=
  local    project_audit_json=
  local    project_path=
  local    project_fullpath=
  local    audit_file=
  local    path_link=
  local    fullpath_link=

  while read -r project_json ; do
    project_id="$( jq -r '.id' <<<"${project_json}" )" || exit 1
    project_path="$( jq -r '.path' <<<"${project_json}" )" || exit 1
    project_fullpath="$( jq -r '.path_with_namespace' <<<"${project_json}" )" || exit 1

    if [ -z "${project_path}" ]; then
      echowarn "*** Error: can not retrieve configuration for project '${project_id}'" >&2
    else
      project_audit_json="$( audit_projects_json_filter <<<"${project_json}" )" || exit 1

      audit_file="$( build_audit_file "${audit_folder}" 'projects_by_id' "${project_id}" )" || exit 1
      path_link="$( build_audit_file "${audit_folder}" 'projects_by_path' "${project_path}" )" || exit 1
      fullpath_link="$( build_audit_file "${audit_folder}" 'projects_by_path_with_namespace' "${project_fullpath}" )" || exit 1

      echoinfo "* audit project ${project_id} / ${project_path}"
      echo "${project_audit_json}" > "${audit_file}"

      mk_relative_link "${audit_file}" "${path_link}"
      mk_relative_link "${audit_file}" "${fullpath_link}"
    fi
  done < <( get_projects_json | jq -c '.[]' )
}

function do_audit {
  local -r audit_folder_home="$1"

  #
  audit_groups_configuration "${audit_folder_home}" || exit 1
  #
  audit_projects_configuration "${audit_folder_home}" || exit 1

  #audit_users_configuration ? "${audit_folder_home}" || exit 1
  #audit_merge_request ? "${audit_folder_home}" || exit 1
  #audit_merge_request_configuration ? "${audit_folder_home}" || exit 1
  #audit_merge_request ? "${audit_folder_home}" || exit 1
}

function main {
  local audit_folder_home=
  local param=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-d'|'--directory')
        audit_folder_home="$1"
        shift
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${audit_folder_home}" ]; then
    echofatal 'audit directory is missing'
    usage >&2
    exit 1
  fi

  # shellcheck disable=SC1090
  source "${PROJECT_HOME}/bin/gitlab-bash-api/init.sh" || exit 1
  # shellcheck disable=SC1090
  source "${GITLAB_BASH_API_PATH}/post.sh" || exit 1
  #source "${GITLAB_BASH_API_PATH}/api/gitlab-bash-api-project.sh"

  if ! is_variable_define_and_not_empty_safe 'GITLAB_AUDIT_FILTER_FOR_GROUP' ; then
    echofatal 'GITLAB_AUDIT_FILTER_FOR_GROUP is not define.'
    exit 1
  fi

  if ! is_variable_define_and_not_empty_safe 'GITLAB_AUDIT_FILTER_FOR_PROJECT' ; then
    echofatal 'GITLAB_AUDIT_FILTER_FOR_PROJECT is not define.'
    exit 1
  fi

  do_audit "${audit_folder_home}"
}

main "$@"
